package com.flyfish.framework.service;

import com.flyfish.framework.domain.AdminUserDetails;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.domain.po.Role;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.utils.CopyUtils;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.collections4.SetUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.*;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class UserDetailsConverter {

    private final DepartmentService departmentService;

    /**
     * 将用户映射为用户详情，查询所有权限code
     *
     * @param user 用户信息
     * @return 结果
     */
    public Mono<UserDetails> mapToUserDetails(IUser user) {
        AdminUserDetails userDetail = new AdminUserDetails();
        CopyUtils.copyProps(user, userDetail);
        userDetail.setAuthorityCodes(Collections.singleton(Department.PUBLIC));
        if (user.getType() == UserType.SUPER_ADMIN) {
            userDetail.setAuthorityCodes(null);
        } else if (CollectionUtils.isNotEmpty(user.getDepartments())) {
            return judgeDeparts(user, userDetail);
        }
        return Mono.just(userDetail);
    }

    /**
     * 判断部门包含关系，默认看不到本部门的数据却可以看到子部门的
     *
     * @param user        用户信息
     * @param userDetails 用户详情
     */
    private Mono<UserDetails> judgeDeparts(IUser user, AdminUserDetails userDetails) {
        // 整合数据权限
        Set<Role.Authority> authorities = user.getRoles().stream().reduce(new HashSet<>(), (result, item) -> {
            // 管理员拥有完全控制权限
            if (BooleanUtils.isTrue(item.getAdmin())) {
                result.add(Role.Authority.ADMIN);
            }
            result.addAll(item.getAuthorities());
            return result;
        }, (a, b) -> a);
        // 根据权限组装查询条件
        Set<String> departs = mergeDeparts(user.getDepartments());
        // 查询所有子部门id
        return departmentService.getSubCodes(departs)
                .map(codes -> {
                    // 取出权限，便于判定
                    boolean admin = authorities.contains(Role.Authority.ADMIN);
                    boolean view = authorities.contains(Role.Authority.VIEW);
                    boolean viewChildren = authorities.contains(Role.Authority.VIEW_CHILDREN);
                    // 全部
                    Set<String> all = new HashSet<>();
                    // 拥有查看本部门的权限
                    if (admin || view) {
                        all.addAll(departs);
                    }
                    // 拥有查看所有下级的权限
                    if (admin || viewChildren) {
                        all.addAll(codes);
                    }
                    // 最终设置权限
                    if (CollectionUtils.isNotEmpty(all)) {
                        userDetails.setAuthorityCodes(SetUtils.union(all, userDetails.getAuthorityCodes()));
                    }
                    userDetails.setVisibleDeparts(SetUtils.union(codes, departs));
                    return userDetails;
                });

    }

    /**
     * 合并部门，有包含关系的部门将被合并
     *
     * @param departments 部门
     * @return 结果
     */
    private Set<String> mergeDeparts(List<Department> departments) {
        Set<String> departs = departments.stream().map(Department::getId)
                .collect(Collectors.toSet());
        List<String> removing = new ArrayList<>();
        for (Department department : departments) {
            if (CollectionUtils.containsAny(department.getParentIds(), departs)) {
                removing.add(department.getId());
            }
        }
        removing.forEach(departs::remove);
        return departs;
    }
}
